package no.ntnu.idatt1002.demo.data.recipes;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

// Method in case we want to check if also the amount is enough for the recipe.
// Recipes without any ingredients can be showed among all ingredients, but not in suggestion based on ingredients
// at hand.
//TODO: Record?
public class Recipe {

    private String name = "";
    private final List<RecipeIngredient> ingredientList = new ArrayList<>();
    private String instructions = "";
    private int missingIngredients;
    private ArrayList<String> missingList = new ArrayList<>();

    public Recipe(String name, String description ) {
        if(name.isBlank() | description.isBlank()) {
            throw new IllegalArgumentException("The recipe must have a name and a description.");
        }
        this.name = name;
        this.instructions = description;
    }


    public String getName() {
        return name;
    }

    public void setName(String name) {
        if(name.isBlank()) {
            throw new IllegalArgumentException("The recipe name cannot be left blank.");
        }
        this.name = name;
    }

    public List<RecipeIngredient> getIngredientList() {
        return ingredientList;
    }

    public String getInstructions() {
        return instructions;
    }

    public void setInstructions(String instructions) {
        if(instructions.isBlank()) {
            throw new IllegalArgumentException("The recipe instructions cannot be left blank.");
        }
        this.instructions = instructions;
    }


 /*   public void addIngredients(ArrayList<RecipeIngredient> ingredients) {
        ingredients.forEach((ingredient) -> this.ingredientList.add(ingredient));
    }*/

    //TODO: Make interface for "collects ingredients" and do deep copy.

    public RecipeIngredient getIngredient(FoodItem ingredientType) {
        if(ingredientType == null) return null;
        return this.getIngredientList().stream()
                .filter((ingredient) -> ingredient.getFoodType() == ingredientType)
                .findFirst().orElse(null);
    }


    public boolean addIngredient(FoodItem ingredientType, double amount, MeasuringUnit unit) {
        if(ingredientList.stream().anyMatch((ingredient) -> ingredient.getFoodType() == ingredientType)) {
            try {
                getIngredient(ingredientType).setAmount(amount);
                getIngredient(ingredientType).setUnit(unit);

            } catch (IllegalArgumentException e) {
                return false;
            }
        } else {
            try {
                this.ingredientList.add(new RecipeIngredient(ingredientType, amount, unit));
            } catch (IllegalArgumentException e) {
                return false;
            }
        }
        return true;
    }

    /**
     * The functionality may be expanded upon in order to also check if the amount is sufficient for the current recipe.
     * @param ingredientsAtHand
     */
    public void updateIngredientStatus(IngredientsAtHand ingredientsAtHand) {
        // Will need a supporting class for converting between units to be accurate.
        if (ingredientsAtHand == null) {
            throw new NullPointerException("The ingredients at hand object must exist");
        } else if (ingredientsAtHand.getIngredientsAtHand().size() < 1) {
            throw new IllegalArgumentException("The collection of ingredients at hand is empty.");
        } else {
            missingIngredients = 0;
            int notMissing = (int) ingredientList.stream().filter((inRecipe) -> ingredientsAtHand.atHand(inRecipe.getFoodType())).count();
            ingredientList.forEach((inRecipe) -> {
                inRecipe.setAtHand(ingredientsAtHand.atHand(inRecipe.getFoodType()));

                if(!ingredientsAtHand.atHand(inRecipe.getFoodType())) {
                    missingList.add(inRecipe.getFoodType().label);

                }
            });
            missingIngredients = ingredientList.size()-notMissing;
        }

    }


    public int getMissingIngredients() {
        return missingIngredients;
    }

    public ArrayList<String> getMissingList() {
        return missingList;
    }

    public String toString() {
        StringBuilder sb = new StringBuilder();

        sb.append(this.name).append("\n");
        this.getIngredientList().forEach((ingredient) -> {
            sb.append(ingredient).append("\n");
        });
        sb.append(this.getInstructions());

        return String.valueOf(sb);
    }


    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (o == null || getClass() != o.getClass()) return false;
        Recipe recipe = (Recipe) o;
        return Objects.equals(name, recipe.name);
    }

    @Override
    public int hashCode() {
        return Objects.hash(name, ingredientList, instructions);
    }
}
